@@ -205,9 +205,14 @@ bool QTiffHandlerPrivate::canRead(QIODevice *device)
 
     // current implementation uses TIFFClientOpen which needs to be
     // able to seek, so sequential devices are not supported
-    QByteArray header = device->peek(4);
-    return header == QByteArray::fromRawData("\x49\x49\x2A\x00", 4)
-           || header == QByteArray::fromRawData("\x4D\x4D\x00\x2A", 4);
+    char h[4];
+    if (device->peek(h, 4) != 4)
+        return false;
+    if ((h[0] == 0x49 && h[1] == 0x49) && (h[2] == 0x2a || h[2] == 0x2b) && h[3] == 0)
+        return true; // Little endian, classic or bigtiff
+    if ((h[0] == 0x4d && h[1] == 0x4d) && h[2] == 0 && (h[3] == 0x2a || h[3] == 0x2b))
+        return true; // Big endian, classic or bigtiff
+    return false;
 }
 
 bool QTiffHandlerPrivate::openForRead(QIODevice *device)
@@ -344,13 +349,7 @@ bool QTiffHandler::read(QImage *image)
 
     QImage::Format format = d->format;
 
-    if (image->size() == d->size && image->format() != format)
-        image->reinterpretAsFormat(format);
-
-    if (image->size() != d->size || image->format() != format)
-        *image = QImage(d->size, format);
-
-    if (image->isNull()) {
+    if (!QImageIOHandler::allocateImage(d->size, format, image)) {
         d->close();
         return false;
     }
@@ -362,7 +361,7 @@ bool QTiffHandler::read(QImage *image)
     // Setup color tables
     if (format == QImage::Format_Mono || format == QImage::Format_Indexed8) {
         if (format == QImage::Format_Mono) {
-            QVector<QRgb> colortable(2);
+            QList<QRgb> colortable(2);
             if (d->photometric == PHOTOMETRIC_MINISBLACK) {
                 colortable[0] = 0xff000000;
                 colortable[1] = 0xffffffff;
@@ -373,7 +372,7 @@ bool QTiffHandler::read(QImage *image)
             image->setColorTable(colortable);
         } else if (format == QImage::Format_Indexed8) {
             const uint16 tableSize = 256;
-            QVector<QRgb> qtColorTable(tableSize);
+            QList<QRgb> qtColorTable(tableSize);
             if (d->grayscale) {
                 for (int i = 0; i<tableSize; ++i) {
                     const int c = (d->photometric == PHOTOMETRIC_MINISBLACK) ? i : (255 - i);
@@ -418,14 +417,22 @@ bool QTiffHandler::read(QImage *image)
             quint32 tileWidth, tileLength;
             TIFFGetField(tiff, TIFFTAG_TILEWIDTH, &tileWidth);
             TIFFGetField(tiff, TIFFTAG_TILELENGTH, &tileLength);
-            uchar *buf = (uchar *)_TIFFmalloc(TIFFTileSize(tiff));
-            if (!tileWidth || !tileLength || !buf) {
-                _TIFFfree(buf);
+            if (!tileWidth || !tileLength || tileWidth % 16 || tileLength % 16) {
                 d->close();
                 return false;
             }
             quint32 byteWidth = (format == QImage::Format_Mono) ? (width + 7)/8 : (width * bytesPerPixel);
             quint32 byteTileWidth = (format == QImage::Format_Mono) ? tileWidth/8 : (tileWidth * bytesPerPixel);
+            tmsize_t byteTileSize = TIFFTileSize(tiff);
+            if (byteTileSize > image->sizeInBytes() || byteTileSize / tileLength < byteTileWidth) {
+                d->close();
+                return false;
+            }
+            uchar *buf = (uchar *)_TIFFmalloc(byteTileSize);
+            if (!buf) {
+                d->close();
+                return false;
+            }
             for (quint32 y = 0; y < height; y += tileLength) {
                 for (quint32 x = 0; x < width; x += tileWidth) {
                     if (TIFFReadTile(tiff, buf, x, y, 0, 0) < 0) {
@@ -443,6 +450,10 @@ bool QTiffHandler::read(QImage *image)
             }
             _TIFFfree(buf);
         } else {
+            if (image->bytesPerLine() < TIFFScanlineSize(tiff)) {
+                d->close();
+                return false;
+            }
             for (uint32 y=0; y<height; ++y) {
                 if (TIFFReadScanline(tiff, image->scanLine(y), y, 0) < 0) {
                     d->close();
@@ -501,7 +512,7 @@ bool QTiffHandler::read(QImage *image)
     return true;
 }
 
-static bool checkGrayscale(const QVector<QRgb> &colorTable)
+static bool checkGrayscale(const QList<QRgb> &colorTable)
 {
     if (colorTable.size() != 256)
         return false;
@@ -515,9 +526,9 @@ static bool checkGrayscale(const QVector<QRgb> &colorTable)
     return true;
 }
 
-static QVector<QRgb> effectiveColorTable(const QImage &image)
+static QList<QRgb> effectiveColorTable(const QImage &image)
 {
-    QVector<QRgb> colors;
+    QList<QRgb> colors;
     switch (image.format()) {
     case QImage::Format_Indexed8:
         colors = image.colorTable();
@@ -647,7 +658,7 @@ bool QTiffHandler::write(const QImage &image)
                || format == QImage::Format_Grayscale8
                || format == QImage::Format_Grayscale16
                || format == QImage::Format_Alpha8) {
-        QVector<QRgb> colorTable = effectiveColorTable(image);
+        QList<QRgb> colorTable = effectiveColorTable(image);
         bool isGrayscale = checkGrayscale(colorTable);
         if (isGrayscale) {
             uint16 photometric = PHOTOMETRIC_MINISBLACK;
@@ -811,13 +822,6 @@ bool QTiffHandler::write(const QImage &image)
     return true;
 }
 
-#if QT_DEPRECATED_SINCE(5, 13)
-QByteArray QTiffHandler::name() const
-{
-    return "tiff";
-}
-#endif
-
 QVariant QTiffHandler::option(ImageOption option) const
 {
     if (option == Size && canRead()) {
@@ -837,7 +841,7 @@ QVariant QTiffHandler::option(ImageOption option) const
 
 void QTiffHandler::setOption(ImageOption option, const QVariant &value)
 {
-    if (option == CompressionRatio && value.type() == QVariant::Int)
+    if (option == CompressionRatio && value.metaType().id() == QMetaType::Int)
         d->compression = qBound(0, value.toInt(), 1);
     if (option == ImageTransformation) {
         int transformation = value.toInt();
@@ -851,8 +855,7 @@ bool QTiffHandler::supportsOption(ImageOption option) const
     return option == CompressionRatio
             || option == Size
             || option == ImageFormat
-            || option == ImageTransformation
-            || option == TransformedByDefault;
+            || option == ImageTransformation;
 }
 
 bool QTiffHandler::jumpToNextImage()
